<?php 
/*
Plugin Name: Automatr
Plugin URI: http://www.poetticcoding.co.uk/plugins/automatr
Description: Automates theme, plugin and core updates
Version: 0.1
Author: Mike Grant
Author URI: http://www.poeticcoding.co.uk
License: GPL2
*/

require_once ( ABSPATH . 'wp-admin/includes/class-wp-upgrader.php' );

class automatr {
	
	private $_settings;
	private $_optionsName = 'automatr';
	private $_optionsGroup = 'automatr-options';
	
	public function __construct() {
		$this->_getSettings();
		if (is_admin()) {
			add_action('admin_init', array($this,'registerOptions'));
			add_action('admin_menu', array($this,'adminMenu'));
		}
		register_activation_hook( __FILE__, array(&$this, 'activatePlugin'));
		register_deactivation_hook( __FILE__, array(&$this, 'deactivatePlugin'));
	}
	
	public function registerOptions() {
		register_setting($this->_optionsGroup, $this->_optionsName);
	}
	
	public function activatePlugin() {
		update_option($this->_optionsName, $this->_settings);
		add_action('automatrPluginUpdater', array($this, 'runPluginUpdate'));
		add_action('automatrThemeUpdater', array($this, 'runThemeUpdate'));
		add_action('automatrCoreUpdater', array($this, 'runCoreUpdate'));
		wp_schedule_event(time(), 'hourly', 'automatrPluginUpdater');
		wp_schedule_event(time(), 'hourly', 'automatrThemeUpdater');
		wp_schedule_event(time(), 'hourly', 'automatrCoreUpdater');
	}
	
	public function deactivatePlugin() {
		delete_option($this->_optionsName);
	}
	
	public function getSetting( $settingName, $default = false ) {
		if (empty($this->_settings)) {
			$this->_getSettings();
		}
		if ( isset($this->_settings[$settingName]) ) {
			return $this->_settings[$settingName];
		} else {
			return $default;
		}
	}
	
	private function _getSettings() {
		if (empty($this->_settings)) {
			$this->_settings = get_option($this->_optionsName);
		}
		if ( !is_array( $this->_settings ) ) {
			$this->_settings = array();
		}
		$defaults = array(
			'theme'	=> 0,
			'plugin'	=> 0,
			'core'	=> 0,
		);
		$this->_settings = wp_parse_args($this->_settings, $defaults);
	}
	
	public function adminMenu() {
		add_options_page(__('Automatr'), __('Automatr'), 'manage_options', 'Automatr', array($this, 'options'));
	}
	
	public function options() {
		?>
		<div class="wrap">
		<h2>Automatr</h2>

		<form method="post" action="options.php">
			<?php settings_fields($this->_optionsGroup); ?>
			<?php do_settings_sections($this->_optionsGroup); ?>
			<table class="form-table">
				<tr valign="top">
				<th scope="row">Automatic Theme Updates: </th>
				<td><input type="checkbox" name="<?php echo $this->_optionsName; ?>[theme]" id="<?php echo $this->_optionsName; ?>_theme" <?php echo (esc_attr($this->_settings['theme']) ? 'checked="checked"' : ''); ?> /></td>
				</tr>
				 
				<tr valign="top">
				<th scope="row">Automatic Plugin Updates: </th>
				<td><input type="checkbox" name="<?php echo $this->_optionsName; ?>[plugin]" id="<?php echo $this->_optionsName; ?>_plugin" <?php echo (esc_attr($this->_settings['plugin']) ? 'checked="checked"' : ''); ?> /></td>
				</tr>
				
				<tr valign="top">
				<th scope="row">Automatic Wordpress Updates: </th>
				<td><input type="checkbox" name="<?php echo $this->_optionsName; ?>[core]" id="<?php echo $this->_optionsName; ?>_core" <?php echo (esc_attr($this->_settings['core']) ? 'checked="checked"' : ''); ?> /></td>
				</tr>
			</table>
			
			<p class="submit">
			<input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
			</p>

		</form>
		</div>
		<?php
	}

	public function runPluginUpdate() {
		if($this->_settings['plugin']) {
			$pluginupdates = $this->preparePluginUpdates();
			if(is_array($pluginupdates)) {
				foreach($pluginupdates as $updater) {
					$this->updatePlugin($updater);
				}	
			}
		}
	}
	
	public function runThemeUpdate() {
		if($this->_settings['theme']) {
			$themeupdates = $this->prepareThemeUpdates();
			if(is_array($themeupdates)) {
				foreach($themeupdates as $updater) {
					$this->updateTheme($updater);
				}	
			}	
		}
	}
	
	public function runCoreUpdate() {
		if($this->_settings['core']) {
			$this->prepareCoreUpdate();
			$this->updateCore();
		}
	}
	
	private function preparePluginUpgrader() {
		$pluginUpgrader = new automatrUpdatePluginSkin();
		return $pluginUpgrader;
	}
	
	private function prepareThemeUpgrader() {
		$themeUpgrader = new automatrUpdateThemeSkin();
		return $themeUpgrader;
	}
	
	private function prepareCoreUpgrader() {
		$coreUpgrader = new automatrUpdateCoreSkin();
		return $coreUpgrader;
	}
	
	private function preparePluginUpdates() {
		// Get a list of all the plugins
		require_once( ABSPATH . '/wp-admin/includes/plugin.php' );
		
		$plugins = get_plugins();
		$active  = get_option( 'active_plugins', array() );
		if ( function_exists( 'get_site_transient' ) ) {
			delete_site_transient( 'update_plugins' );
		} else {
			delete_transient( 'update_plugins' );
		}
		wp_update_plugins();
		if( function_exists( 'get_site_transient' ) && $transient = get_site_transient( 'update_plugins' ) ) {
			$current = $transient;
		} elseif( $transient = get_transient( 'update_plugins' ) ) {
			$current = $transient;
		} else {
			$current = get_option( 'update_plugins' );
		}
		
		foreach ( (array) $plugins as $plugin_file => $plugin ) {
			$new_version = isset( $current->response[$plugin_file] ) ? $current->response[$plugin_file]->new_version : null;
			if ( $new_version ) {
				$updates[] = $plugin_file;		
			}
		}
		return $updates;
	}
	
	private function prepareThemeUpdates() {
		require_once( ABSPATH . '/wp-admin/includes/theme.php' );
		$themes = get_themes();
		$active  = get_option( 'current_theme' );
		wp_update_themes();
		if ( function_exists( 'get_site_transient' ) && $transient = get_site_transient( 'update_themes' ) ) {
			$current = $transient;
		} elseif ( $transient = get_transient( 'update_themes' ) ) {
			$current = $transient;
		} else {
			$current = get_option( 'update_themes' );
		}
		foreach ( (array) $themes as $theme ) {
			$new_version = isset( $current->response[$theme['Template']] ) ? $current->response[$theme['Template']]['new_version'] : null;
			if($new_version) {
				$updates[] = $theme['Name'];
			}
		}
		return $updates;
	}

	private function prepareCoreUpdate() {
		// force refresh
		wp_version_check();
		$updates = get_core_updates();
		return $updates;
	}
	
	private function updatePlugin($plugin) {
		include_once ( ABSPATH . 'wp-admin/includes/admin.php' );
		$skin = $this->preparePluginUpgrader();
		$upgrader = new Plugin_Upgrader( $skin );
		$is_active = is_plugin_active( $plugin );
		wp_update_plugins();
		ob_start();
		$result = $upgrader->upgrade( $plugin );
		$data = ob_get_contents();
		ob_clean();
		if ( ( ! $result && ! is_null( $result ) ) || $data ) {
			return array( 'status' => 'error', 'error' => 'file_permissions_error' );
		} elseif ( is_wp_error( $result ) ) {
			return array( 'status' => 'error', 'error' => $result->get_error_code() );
		}
		if ( $skin->error ) {
			return array( 'status' => 'error', 'error' => $skin->error );
		}
		if ( $is_active ) {
			$url = add_query_arg( 'wpr_api_key', $_GET['wpr_api_key'], get_bloginfo( 'url' ) );
			$url = add_query_arg( 'actions', 'activate_plugin', $url );
			$url = add_query_arg( 'plugin', $plugin, $url );
			$request = wp_remote_get( $url );
		}
		if ( is_wp_error( $request ) ) {
			return array( 'status' => 'error', 'error' => $request->get_error_code() );
		}
		$body = wp_remote_retrieve_body( $request );
		if ( ! $json = @json_decode( $body ) ) {
			return array( 'status' => 'error', 'error' => 'The plugin was updated, but failed to re-activate.' );
		}
		$json = $json->activate_plugin;
		if ( empty( $json->status ) ) {
			return array( 'status' => 'error', 'error' => 'The plugin was updated, but failed to re-activate. The activation reuest returned no response' );
		}
		if ( $json->status != 'success' ) {
			return array( 'status' => 'error', 'error' => 'The plugin was updated, but failed to re-activate. The activation reuest returned response: ' . $json->status );
		}
		return array( 'status' => 'success' );
	}

	private function updateTheme( $theme ) {
		include_once ( ABSPATH . 'wp-admin/includes/admin.php' );
		$skin = $this->prepareThemeUpgrader();
		$upgrader = new Theme_Upgrader( $skin );
		ob_start();
		$result = $upgrader->upgrade( $theme );
		$data = ob_get_contents();
		ob_clean();
		if ( ( ! $result && ! is_null( $result ) ) || $data ) {
			return array( 'status' => 'error', 'error' => 'file_permissions_error' );
		} elseif ( is_wp_error( $result ) ) {
			return array( 'status' => 'error', 'error' => $result->get_error_code() );
		}
		if ( $skin->error ) {
			return array( 'status' => 'error', 'error' => $skin->error );
		}
		return array( 'status' => 'success' );
	}

	private function pluginActivator($plugin) {
		include_once ABSPATH . 'wp-admin/includes/plugin.php';
		$result = activate_plugin( $plugin );
		if ( is_wp_error( $result ) )
			return array( 'status' => 'error', 'error' => $result->get_error_code() );
		return array( 'status' => 'success' );
	}
	
	private function updateCore() {
		include_once ( ABSPATH . 'wp-admin/includes/admin.php' );
		include_once ( ABSPATH . 'wp-admin/includes/upgrade.php' );
		include_once ( ABSPATH . 'wp-includes/update.php' );
		// force refresh
		wp_version_check();
	
		$updates = get_core_updates();
	
		if ( is_wp_error( $updates ) || ! $updates )
			return new WP_Error( 'no-update-available' );
	
		$update = reset( $updates );
	
		if ( ! $update )
			return new WP_Error( 'no-update-available' );
	
		$skin = new automatrUpdateCoreSkin();
	
		$upgrader = new Core_Upgrader( $skin );
		$result = $upgrader->upgrade($update);
	
		if ( is_wp_error( $result ) )
			return $result;
	
		global $wp_current_db_version, $wp_db_version;
	
		// we have to include version.php so $wp_db_version
		// will take the version of the updated version of wordpress
		require( ABSPATH . WPINC . '/version.php' );
	
		wp_upgrade();
	
		return true;
	}
}


class automatrUpdatePluginSkin extends Plugin_Installer_Skin {
	var $feedback;
	var $error;
	function error( $error ) {
		$this->error = $error;
	}
	function feedback( $feedback ) {
		$this->feedback = $feedback;
	}
	function before() {
	}
	function after() {
	}
	function header() {
	}
	function footer() {
	}
}

class automatrUpdateThemeSkin extends Theme_Installer_Skin {
	var $feedback;
	var $error;
	function error( $error ) {
		$this->error = $error;
	}
	function feedback( $feedback ) {
		$this->feedback = $feedback;
	}
	function before() {
	}
	function after() {
	}
	function header() {
	}
	function footer() {
	}
}

class automatrUpdateCoreSkin extends WP_Upgrader_Skin {
	var $feedback;
	var $error;
	function error( $error ) {
		$this->error = $error;
	}
	function feedback( $feedback ) {
		$this->feedback = $feedback;
	}
	function before() { }

	function after() { }

	function header() { }

	function footer() { }

}

$automatr = new automatr();
?>